@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1999, 2002, 2003  Free Software Foundation, Inc.
+ * Copyright (C) 1999, 2002, 2003, 2004  Free Software Foundation, Inc.
  * 
  * This file is part of GNU libmatheval
  * 
@@ -39,6 +39,14 @@
 #define EXIT_FAILURE  1
 #endif
 
+#ifndef FALSE
+#define FALSE 0
+#endif
+
+#ifndef TRUE
+#define TRUE 1
+#endif
+
 #include "xmalloc.h"
 
 #endif
