@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002, 2003  Free Software Foundation, Inc.
+ * Copyright (C) 2002, 2003, 2004  Free Software Foundation, Inc.
  * 
  * This file is part of GNU libmatheval
  * 
@@ -22,15 +22,17 @@
 #include "config.h"
 #endif
 
+#include "_stdint.h"
 #include "common.h"
 #include "matheval.h"
 
 /*
- * Wrapper for evaluator_create() function.  Evaluator objects will be passed
- * between Fortran and C as integers instead of void pointers, thus equal
- * length of int and void* types is assumed throughout this interface.
+ * Wrapper for evaluator_create() function.  Evaluator objects will be
+ * passed between Fortran and C as 64-bit integers instead of void
+ * pointers, thus size of void* is assumed to be less or equal of 64
+ * bits throughout this interface.
  */
-int
+int64_t
 evaluator_create__(char *string, int length)
 {
 	char           *stringz;/* Zero-terminated string representing
@@ -54,19 +56,19 @@ evaluator_create__(char *string, int length)
 	 */
 	XFREE(stringz);
 
-	return evaluator;
+	return (int64_t) evaluator;
 }
 
 /* Wrapper for evaluator_destroy() function.  */
 void
-evaluator_destroy__(int *evaluator)
+evaluator_destroy__(int64_t *evaluator)
 {
 	evaluator_destroy((void *)*evaluator);
 }
 
 /* Wrapper for evaluator_evaluate() function.  */
 double
-evaluator_evaluate__(int *evaluator, int *count, char *names, double *values, int length)
+evaluator_evaluate__(int64_t *evaluator, int *count, char *names, double *values, int length)
 {
 	char          **names_copy;	/* Copy of variable names.  Names are
 					 * passed in single string from
@@ -81,7 +83,6 @@ evaluator_evaluate__(int *evaluator, int *count, char *names, double *values, in
 	 * with each string containing single name.
 	 */
 	names_copy = XMALLOC(char *, *count);
-
 	for (i = j = 0; i < *count && j < length; i++, j += n) {
 		for (; names[j] == ' '; j++);
 		for (n = 1; j + n < length && !(names[j + n] == ' '); n++);
@@ -105,45 +106,93 @@ evaluator_evaluate__(int *evaluator, int *count, char *names, double *values, in
 	return result;
 }
 
-/* Wrapper for evaluator_calculate_length() function.  */
+/* First in pair of wrappers for evaluator_get_string() function.  */
 int
-evaluator_calculate_length__(int *evaluator)
+evaluator_get_string_length__(int64_t *evaluator)
 {
-	return evaluator_calculate_length((void *)*evaluator);
+        /*
+         * Return length of evaluator textual respresentation.
+         */
+        return strlen(evaluator_get_string((void*) *evaluator));
 }
 
-/* Wrapper for evaluator_write() function.  */
+/* Second in pair of wrappers for evaluator_get_string() function.  */
 void
-evaluator_write__(int *evaluator, char *string, int length)
+evaluator_get_string_chars__(int64_t *evaluator, char *string, int length)
 {
-	char           *stringz;/* Zero-terminated string for evaluator
-				 * textual representation. */
-
 	/*
-	 * Allocate string of appropriate length to write evaluator to.
-	 */
-	stringz = XMALLOC(char, length + 1);
-
-	/*
-	 * Call evaluator_write() function.
-	 */
-	evaluator_write((void *)*evaluator, stringz);
-
-	/*
-	 * Copy evaluator textual representation to string passed from
+	 * Copy evaluator textual respresentation to string passed from
 	 * Fortran code.
 	 */
-	memcpy(string, stringz, length * sizeof(char));
+	memcpy(string, evaluator_get_string((void *)*evaluator), length * sizeof(char));
+}
 
-	/*
-	 * Free string used for evaluator textual representation.
-	 */
-	XFREE(stringz);
+/* First in pair of wrappers for evaluator_get_variables() function.  */
+int
+evaluator_get_variables_length__(int64_t *evaluator)
+{
+        char          **names;  /* Array with variable names. */
+        int             count;  /* Number of elements in above array. */
+        int             length; /* Length of string with concatenated
+                                 * variable names */
+        int             i;      /* Loop counter. */
+
+        /*
+	 * Get array of strings with variable names.
+         */
+        evaluator_get_variables((void*) *evaluator, &names, &count);
+
+        /* 
+         * Calculate length of string with concatenated names from above
+         * array.
+         */
+        length = 0;
+        for (i = 0; i < count; i++) {
+                if (i != 0)
+                        length++;
+                length += strlen(names[i]);
+        }
+
+        return length;
+}
+
+/* Second in pair of wrappers for evaluator_get_variables() function.  */
+void
+evaluator_get_variables_chars__(int64_t *evaluator, char *string, int length)
+{
+        char          **names;  /* Array with variable names. */
+        int             count;  /* Number of elements in above array. */
+        int             n;      /* Number of characters to be copied
+                                 * from current variable name to string
+                                 * with concatenated variable names. */
+        int             i;      /* Loop counter. */
+
+        /*
+	 * Get array of strings with variable names.
+         */
+        evaluator_get_variables((void*) *evaluator, &names, &count);
+
+        /*
+         * Concatenate variable names from above array into string
+         * passed from Fortran code.
+         */
+        for (i = 0; i < count; i++) {
+                if (i != 0 && length > 0) {
+                        *(string++) = ' ';
+                        length--;
+                }
+                n = strlen(names[i]);
+                if (n > length)
+                        n = length;
+                memcpy(string, names[i], n * sizeof(char));
+                string += n;
+                length -= n;
+        }
 }
 
 /* Wrapper for evaluator_derivative() function.  */
 int
-evaluator_derivative__(int *evaluator, char *name, int length)
+evaluator_derivative__(int64_t *evaluator, char *name, int length)
 {
 	char           *stringz;/* Zero terminated string containing
 				 * derivation variable name.  */
@@ -172,42 +221,42 @@ evaluator_derivative__(int *evaluator, char *name, int length)
 
 /* Wrapper for evaluator_evaluate_x() function.  */
 double
-evaluator_evaluate_x__(int *evaluator, double *x)
+evaluator_evaluate_x__(int64_t *evaluator, double *x)
 {
 	return evaluator_evaluate_x((void *)*evaluator, *x);
 }
 
 /* Wrapper for evaluator_evaluate_x_y() function.  */
 double
-evaluator_evaluate_x_y__(int *evaluator, double *x, double *y)
+evaluator_evaluate_x_y__(int64_t *evaluator, double *x, double *y)
 {
 	return evaluator_evaluate_x_y((void *)*evaluator, *x, *y);
 }
 
 /* Wrapper for evaluator_evaluate_x_y_z() function.  */
 double
-evaluator_evaluate_x_y_z__(int *evaluator, double *x, double *y, double *z)
+evaluator_evaluate_x_y_z__(int64_t *evaluator, double *x, double *y, double *z)
 {
 	return evaluator_evaluate_x_y_z((void *)*evaluator, *x, *y, *z);
 }
 
 /* Wrapper for evaluator_derivative_x() function.  */
 int
-evaluator_derivative_x__(int *evaluator)
+evaluator_derivative_x__(int64_t *evaluator)
 {
 	return (int)evaluator_derivative_x((void *)*evaluator);
 }
 
 /* Wrapper for evaluator_derivative_y() function.  */
 int
-evaluator_derivative_y__(int *evaluator)
+evaluator_derivative_y__(int64_t *evaluator)
 {
 	return (int)evaluator_derivative_y((void *)*evaluator);
 }
 
 /* Wrapper for evaluator_derivative_z() function.  */
 int
-evaluator_derivative_z__(int *evaluator)
+evaluator_derivative_z__(int64_t *evaluator)
 {
 	return (int)evaluator_derivative_z((void *)*evaluator);
 }
