@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1999, 2002, 2003  Free Software Foundation, Inc.
+ * Copyright (C) 1999, 2002, 2003, 2004  Free Software Foundation, Inc.
  * 
  * This file is part of GNU libmatheval
  * 
@@ -423,8 +423,18 @@ node_derivative(Node * node, char *name, SymbolTable * symbol_table)
 		/*
 		 * Apply rule of cotangent function derivative.
 		 */
-		else if (!strcmp(node->data.function.record->name, "ctan"))
+		else if (!strcmp(node->data.function.record->name, "cot"))
 			return node_create('u', '-', node_create('b', '/', node_derivative(node->data.function.child, name, symbol_table), node_create('b', '^', node_create('f', symbol_table_lookup(symbol_table, "sin"), node_copy(node->data.function.child)), node_create('c', 2.0))));
+                /*
+		 * Apply rule of secant function derivative.
+		 */
+		else if (!strcmp(node->data.function.record->name, "sec"))
+                        return node_create('b', '*', node_derivative(node->data.function.child, name, symbol_table), node_create('b', '*', node_create('f', symbol_table_lookup(symbol_table, "sec"), node_copy(node->data.function.child)), node_create('f', symbol_table_lookup(symbol_table, "tan"), node_copy(node->data.function.child))));
+                /*
+		 * Apply rule of cosecant function derivative.
+		 */
+		else if (!strcmp(node->data.function.record->name, "csc"))
+                        return node_create('b', '*', node_derivative(node->data.function.child, name, symbol_table), node_create('u', '-', node_create('b', '*', node_create('f', symbol_table_lookup(symbol_table, "cot"), node_copy(node->data.function.child)), node_create('f', symbol_table_lookup(symbol_table, "csc"), node_copy(node->data.function.child)))));
 		/*
 		 * Apply rule of inverse sine function derivative.
 		 */
@@ -443,8 +453,18 @@ node_derivative(Node * node, char *name, SymbolTable * symbol_table)
 		/*
 		 * Apply rule of inverse cotanget function derivative.
 		 */
-		else if (!strcmp(node->data.function.record->name, "actan"))
+		else if (!strcmp(node->data.function.record->name, "acot"))
 			return node_create('u', '-', node_create('b', '/', node_derivative(node->data.function.child, name, symbol_table), node_create('b', '+', node_create('c', 1.0), node_create('b', '^', node_copy(node->data.function.child), node_create('c', 2.0)))));
+                /*
+		 * Apply rule of inverse secant function derivative.
+		 */
+		else if (!strcmp(node->data.function.record->name, "asec"))
+			return node_create('b', '*', node_derivative(node->data.function.child, name, symbol_table), node_create('b', '/', node_create('c', 1.0), node_create('b', '*', node_create('b', '^', node_copy(node->data.function.child), node_create('c', 2.0)), node_create('f', symbol_table_lookup(symbol_table, "sqrt"), node_create('b', '-', node_create('c', 1.0), node_create('b', '/', node_create('c', 1.0), node_create('b', '^', node_copy(node->data.function.child), node_create('c', 2.0))))))));
+                /*
+		 * Apply rule of inverse cosecant function derivative.
+		 */
+		else if (!strcmp(node->data.function.record->name, "acsc"))
+			return node_create('b', '*', node_derivative(node->data.function.child, name, symbol_table), node_create('u', '-', node_create('b', '/', node_create('c', 1.0), node_create('b', '*', node_create('b', '^', node_copy(node->data.function.child), node_create('c', 2.0)), node_create('f', symbol_table_lookup(symbol_table, "sqrt"), node_create('b', '-', node_create('c', 1.0), node_create('b', '/', node_create('c', 1.0), node_create('b', '^', node_copy(node->data.function.child), node_create('c', 2.0)))))))));
 		/*
 		 * Apply rule of hyperbolic sine function derivative.
 		 */
@@ -463,8 +483,18 @@ node_derivative(Node * node, char *name, SymbolTable * symbol_table)
 		/*
 		 * Apply rule of hyperbolic cotangent function derivative.
 		 */
-		else if (!strcmp(node->data.function.record->name, "ctanh"))
+		else if (!strcmp(node->data.function.record->name, "coth"))
 			return node_create('u', '-', node_create('b', '/', node_derivative(node->data.function.child, name, symbol_table), node_create('b', '^', node_create('f', symbol_table_lookup(symbol_table, "sinh"), node_copy(node->data.function.child)), node_create('c', 2.0))));
+                /*
+		 * Apply rule of hyperbolic secant function derivative.
+		 */
+		else if (!strcmp(node->data.function.record->name, "sech"))
+                        return node_create('b', '*', node_derivative(node->data.function.child, name, symbol_table), node_create('u', '-', node_create('b', '*', node_create('f', symbol_table_lookup(symbol_table, "sech"), node_copy(node->data.function.child)), node_create('f', symbol_table_lookup(symbol_table, "tanh"), node_copy(node->data.function.child)))));
+                /*
+		 * Apply rule of hyperbolic cosecant function derivative.
+		 */
+		else if (!strcmp(node->data.function.record->name, "csch"))
+                        return node_create('b', '*', node_derivative(node->data.function.child, name, symbol_table), node_create('u', '-', node_create('b', '*', node_create('f', symbol_table_lookup(symbol_table, "coth"), node_copy(node->data.function.child)), node_create('f', symbol_table_lookup(symbol_table, "csch"), node_copy(node->data.function.child)))));
 		/*
 		 * Apply rule of inverse hyperbolic sine function derivative.
 		 */
@@ -486,13 +516,40 @@ node_derivative(Node * node, char *name, SymbolTable * symbol_table)
 		 * Apply rule of inverse hyperbolic cotangent function
 		 * derivative.
 		 */
-		else if (!strcmp(node->data.function.record->name, "actanh"))
+		else if (!strcmp(node->data.function.record->name, "acoth"))
 			return node_create('b', '/', node_derivative(node->data.function.child, name, symbol_table), node_create('b', '-', node_create('b', '^', node_copy(node->data.function.child), node_create('c', 2.0)), node_create('c', 1.0)));
+                /*
+		 * Apply rule of inverse hyperbolic secant function
+		 * derivative.
+		 */
+		else if (!strcmp(node->data.function.record->name, "asech"))
+			return node_create('b', '*', node_derivative(node->data.function.child, name, symbol_table), node_create('u', '-', node_create('b', '*', node_create('b', '/', node_create('c', 1.0), node_create('b', '*', node_copy(node->data.function.child), node_create('f', symbol_table_lookup(symbol_table, "sqrt"), node_create('b', '-', node_create('c', 1.0), node_copy(node->data.function.child))))), node_create('f', symbol_table_lookup(symbol_table, "sqrt"), node_create('b', '/', node_create('c', 1.0), node_create('b', '+', node_create('c', 1.0), node_copy(node->data.function.child)))))));
+                /*
+		 * Apply rule of inverse hyperbolic cosecant function
+		 * derivative.
+		 */
+		else if (!strcmp(node->data.function.record->name, "acsch"))
+                        return node_create('b', '*', node_derivative(node->data.function.child, name, symbol_table), node_create('u', '-', node_create('b', '/', node_create('c', 1.0), node_create('b', '*', node_create('b', '^', node_copy(node->data.function.child), node_create('c', 2.0)), node_create('f', symbol_table_lookup(symbol_table, "sqrt"), node_create('b', '+', node_create('c', 1.0), node_create('b', '/', node_create('c', 1.0), node_create('b', '^', node_copy(node->data.function.child), node_create('c', 2.0)))))))));
 		/*
 		 * Apply rule of absolute value function derivative.
 		 */
-		else if (!strcmp(node->data.function.record->name, "fabs"))
-			return node_create('b', '/', node_create('b', '*', node_derivative(node->data.function.child, name, symbol_table), node_copy(node->data.function.child)), node_create('f', symbol_table_lookup(symbol_table, "sqrt"), node_create('b', '^', node_copy(node->data.function.child), node_create('c', 2.0))));
+		else if (!strcmp(node->data.function.record->name, "abs"))
+			return node_create('b', '*', node_derivative(node->data.function.child, name, symbol_table), node_create('b', '-', node_create('b', '*', node_create('c', 2.0), node_create('f', symbol_table_lookup(symbol_table, "step"), node_copy(node->data.function.child))), node_create('c', 1.0)));
+                /*
+		 * Apply rule of step function derivative.
+		 */
+		else if (!strcmp(node->data.function.record->name, "step"))
+			return node_create('b', '*', node_derivative(node->data.function.child, name, symbol_table), node_create('f', symbol_table_lookup(symbol_table, "delta"), node_copy(node->data.function.child)));
+                /*
+		 * Apply rule of delta function derivative.
+		 */
+		else if (!strcmp(node->data.function.record->name, "delta"))
+			return node_create('b', '*', node_derivative(node->data.function.child, name, symbol_table), node_create('f', symbol_table_lookup(symbol_table, "nandelta"), node_copy(node->data.function.child)));
+                /*
+		 * Apply rule of nandelta function derivative.
+		 */
+		else if (!strcmp(node->data.function.record->name, "nandelta"))
+			return node_create('b', '*', node_derivative(node->data.function.child, name, symbol_table), node_create('f', symbol_table_lookup(symbol_table, "nandelta"), node_copy(node->data.function.child)));
 
 	case 'u':
 		switch (node->data.un_op.operation) {
@@ -537,7 +594,7 @@ node_derivative(Node * node, char *name, SymbolTable * symbol_table)
 			if (node->data.bin_op.right->type == 'c')
 				return node_create('b', '*', node_create('b', '*', node_create('c', node->data.bin_op.right->data.constant), node_derivative(node->data.bin_op.left, name, symbol_table)), node_create('b', '^', node_copy(node->data.bin_op.left), node_create('c', node->data.bin_op.right->data.constant - 1.0)));
 			/*
-			 * Otherwise, apply logaritmhic derivative rule:
+			 * Otherwise, apply logarithmic derivative rule:
 			 * (log(f^g))'=(f^g)'/f^g =>
 			 * (f^g)'=f^g*(log(f^g))'=f^g*(g*log(f))'
 			 */
@@ -553,8 +610,35 @@ node_derivative(Node * node, char *name, SymbolTable * symbol_table)
 	}
 }
 
+void
+node_flag_variables(Node * node)
+{
+        /*
+	 * According to node type, flag variable in symbol table or
+	 * proceed with calling function recursively on node children.
+	 */
+        switch (node->type) {
+        case 'v':
+                node->data.variable->flag = TRUE;
+                break;
+
+        case 'f':
+                node_flag_variables(node->data.function.child);
+                break;
+
+        case 'u':
+                node_flag_variables(node->data.un_op.child);
+                break;
+
+        case 'b':
+                node_flag_variables(node->data.bin_op.left);
+                node_flag_variables(node->data.bin_op.right);
+                break;
+        }
+}
+
 int
-node_calculate_length(Node * node)
+node_get_length(Node * node)
 {
         FILE           *file;	/* Temporary file. */
         int             count;	/* Count of bytes written to above file. */
@@ -571,7 +655,7 @@ node_calculate_length(Node * node)
 			length += 1;
 
                 file = tmpfile();
-                if (file != NULL) {
+                if (file) {
                         if ((count = fprintf(file, "%g", node->data.constant)) >= 0)
                                 length += count;
                         fclose(file);
@@ -585,14 +669,14 @@ node_calculate_length(Node * node)
 		return strlen(node->data.variable->name);
 
 	case 'f':
-		return strlen(node->data.function.record->name) + 1 + node_calculate_length(node->data.function.child) + 1;
+		return strlen(node->data.function.record->name) + 1 + node_get_length(node->data.function.child) + 1;
 		break;
 
 	case 'u':
-		return 1 + 1 + node_calculate_length(node->data.un_op.child) + 1;
+		return 1 + 1 + node_get_length(node->data.un_op.child) + 1;
 
 	case 'b':
-		return 1 + node_calculate_length(node->data.bin_op.left) + 1 + node_calculate_length(node->data.bin_op.right) + 1;
+		return 1 + node_get_length(node->data.bin_op.left) + 1 + node_get_length(node->data.bin_op.right) + 1;
 	}
 
 	return 0;
